@@ -15,8 +15,9 @@ from zipfile import ZipFile
 from cognite.client import utils
 from cognite.client._api_client import APIClient
 from cognite.client._constants import LIST_LIMIT_CEILING, LIST_LIMIT_DEFAULT
-from cognite.client.credentials import OAuthClientCredentials, Token
+from cognite.client.credentials import APIKey, OAuthClientCertificate
 from cognite.client.data_classes import (
+    ClientCredentials,
     Function,
     FunctionCall,
     FunctionCallList,
@@ -31,7 +32,7 @@ from cognite.client.data_classes import (
 )
 from cognite.client.data_classes.files import FileMetadata
 from cognite.client.data_classes.functions import FunctionCallsFilter, FunctionsStatus
-from cognite.client.exceptions import CogniteAPIError
+from cognite.client.exceptions import CogniteAuthError
 from cognite.client.utils._auxiliary import is_unlimited
 from cognite.client.utils._identifier import IdentifierSequence, SingletonIdentifierSequence
 
@@ -386,14 +387,7 @@ class FunctionsAPI(APIClient):
         """
         identifier = IdentifierSequence.load(ids=id, external_ids=external_id).as_singleton()
         id = _get_function_internal_id(self._cognite_client, identifier)
-
-        # Case 1: Client credentials inferred from the instantiated client.
-        # Case 2: Token on behalf of the user. We use token exchange.
-        nonce = None
-        if _using_client_credential_flow(self._cognite_client):
-            nonce = _use_client_credentials(self._cognite_client, client_credentials=None)
-        elif _using_token_exchange_flow(self._cognite_client):
-            nonce = _use_token_exchange(self._cognite_client)
+        nonce = _create_session_and_return_nonce(self._cognite_client)
 
         if data is None:
             data = {}
@@ -541,62 +535,19 @@ class FunctionsAPI(APIClient):
         return FunctionsStatus._load(res.json())
 
 
-def _use_client_credentials(
-    cognite_client: CogniteClient,
-    client_credentials: Optional[Dict] = None,
-) -> str:
-    """
-    If client_credentials is passed, will use those, otherwise will implicitly use those the client was instantiated
-    with
-    Args:
-        client_credentials: a dictionary containing:
-            client_id
-            client_secret
+def _create_session_and_return_nonce(
+    client: CogniteClient,
+    client_credentials: Union[Dict, ClientCredentials, None] = None,
+) -> Optional[str]:
+    if client_credentials is None:
+        if isinstance(client._config.credentials, APIKey):
+            return None
+        elif isinstance(client._config.credentials, OAuthClientCertificate):
+            raise CogniteAuthError("Client certificate credentials is not supported with the Functions API")
 
-    Returns:
-        nonce (optional, str): a nonce if able to obtain, otherwise raises CogniteAPIError.
-
-    """
-
-    if client_credentials:
-        client_id = client_credentials["client_id"]
-        client_secret = client_credentials["client_secret"]
-    else:
-        assert isinstance(cognite_client.config.credentials, OAuthClientCredentials)
-        client_id = cognite_client.config.credentials.client_id
-        client_secret = cognite_client.config.credentials.client_secret
-
-    session_url = f"/api/v1/projects/{cognite_client.config.project}/sessions"
-    payload = {"items": [{"clientId": client_id, "clientSecret": client_secret}]}
-    try:
-        res = cognite_client.post(session_url, json=payload)
-        nonce = res.json()["items"][0]["nonce"]
-        return nonce
-    except CogniteAPIError as e:
-        raise CogniteAPIError("Failed to create session using client credentials flow.", 403) from e
-
-
-def _use_token_exchange(
-    cognite_client: CogniteClient,
-) -> str:
-    session_url = f"/api/v1/projects/{cognite_client.config.project}/sessions"
-    payload = {"items": [{"tokenExchange": True}]}
-    try:
-        res = cognite_client.post(url=session_url, json=payload)
-        nonce = res.json()["items"][0]["nonce"]
-        return nonce
-    except CogniteAPIError as e:
-        raise CogniteAPIError("Failed to create session using token exchange flow.", 403) from e
-
-
-def _using_token_exchange_flow(cognite_client: CogniteClient) -> bool:
-    """Determine whether the Cognite client is configured with a token or token factory."""
-    return isinstance(cognite_client.config.credentials, Token)
-
-
-def _using_client_credential_flow(cognite_client: CogniteClient) -> bool:
-    """Determine whether the Cognite client is configured for client-credential flow."""
-    return isinstance(cognite_client.config.credentials, OAuthClientCredentials)
+    elif isinstance(client_credentials, dict):
+        client_credentials = ClientCredentials(client_credentials["client_id"], client_credentials["client_secret"])
+    return client.iam.sessions.create(client_credentials).nonce
 
 
 def convert_file_path_to_module_path(file_path: str) -> str:
@@ -732,10 +683,6 @@ def _get_fn_docstring_requirements(fn: Callable) -> List[str]:
 class FunctionCallsAPI(APIClient):
     _LIST_CLASS = FunctionCallList
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-        self._cognite_client: CogniteClient = cast("CogniteClient", self._cognite_client)
-
     def list(
         self,
         function_id: Optional[int] = None,
@@ -909,10 +856,6 @@ class FunctionSchedulesAPI(APIClient):
     _RESOURCE_PATH = "/functions/schedules"
     _LIST_CLASS = FunctionSchedulesList
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-        self._cognite_client: CogniteClient = cast("CogniteClient", self._cognite_client)
-
     def retrieve(self, id: int) -> Union[FunctionSchedule, FunctionSchedulesList, None]:
         """`Retrieve a single function schedule by id. <https://docs.cognite.com/api/v1/#operation/byIdsFunctionSchedules>`_
 
@@ -1000,7 +943,7 @@ class FunctionSchedulesAPI(APIClient):
         cron_expression: str,
         function_id: Optional[int] = None,
         function_external_id: Optional[str] = None,
-        client_credentials: Optional[Dict] = None,
+        client_credentials: Union[Dict, ClientCredentials, None] = None,
         description: str = "",
         data: Optional[Dict] = None,
     ) -> FunctionSchedule:
@@ -1012,7 +955,7 @@ class FunctionSchedulesAPI(APIClient):
             function_external_id (optional, str): External id of the function. This is deprecated and cannot be used together with client_credentials.
             description (str): Description of the schedule.
             cron_expression (str): Cron expression.
-            client_credentials: (optional, Dict): Dictionary containing client credentials:
+            client_credentials: (optional, ClientCredentials, Dict): Instance of ClientCredentials or a dictionary containing client credentials:
                 client_id
                 client_secret
             data (optional, Dict): Data to be passed to the scheduled run. **WARNING:** Secrets or other confidential information should not be passed via this argument. There is a dedicated `secrets` argument in FunctionsAPI.create() for this purpose.
@@ -1025,12 +968,13 @@ class FunctionSchedulesAPI(APIClient):
             Create function schedule::
 
                 >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import ClientCredentials
                 >>> c = CogniteClient()
                 >>> schedule = c.functions.schedules.create(
                 ...     name= "My schedule",
                 ...     function_id=123,
                 ...     cron_expression="*/5 * * * *",
-                ...     client_credentials={"client_id": "...", "client_secret": "..."},
+                ...     client_credentials=ClientCredentials("my-client-id", "my-client-secret"),
                 ...     description="This schedule does magic stuff."
                 ... )
 
@@ -1038,9 +982,10 @@ class FunctionSchedulesAPI(APIClient):
         _get_function_identifier(function_id, function_external_id)
 
         nonce = None
-        if client_credentials:
-            assert function_id is not None, "function_id must be set when creating a schedule with client_credentials."
-            nonce = _use_client_credentials(self._cognite_client, client_credentials)
+        if client_credentials is not None:
+            if function_id is None:
+                raise ValueError("When passing 'client_credentials', 'function_id' must be set")
+            nonce = _create_session_and_return_nonce(self._cognite_client, client_credentials)
 
         body: Dict[str, List[Dict[str, Union[str, int, None, Dict]]]] = {
             "items": [
@@ -1058,9 +1003,8 @@ class FunctionSchedulesAPI(APIClient):
         if data:
             body["items"][0]["data"] = data
 
-        url = "/functions/schedules"
-        res = self._post(url, json=body)
-        return FunctionSchedule._load(res.json()["items"][0])
+        res = self._post(self._RESOURCE_PATH, json=body)
+        return FunctionSchedule._load(res.json()["items"][0], cognite_client=self._cognite_client)
 
     def delete(self, id: int) -> None:
         """`Delete a schedule associated with a specific project. <https://docs.cognite.com/api/v1/#operation/deleteFunctionSchedules>`_
@@ -1081,26 +1025,27 @@ class FunctionSchedulesAPI(APIClient):
 
         """
         body = {"items": [{"id": id}]}
-        url = "/functions/schedules/delete"
+        url = f"{self._RESOURCE_PATH}/delete"
         self._post(url, json=body)
 
-    def get_input_data(self, id: int) -> Dict:
+    def get_input_data(self, id: int) -> Optional[Dict]:
         """`Retrieve the input data to the associated function. <https://docs.cognite.com/api/v1/#operation/getFunctionScheduleInputData>`_
         Args:
             id (int): Id of the schedule
 
         Returns:
-            Input data to the associated function. This data is passed
+            Optional[Dict]: Input data to the associated function or None if not set. This data is passed
             deserialized into the function through the data argument.
+
         Examples:
 
             Get schedule input data::
 
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
-                >>> c.functions.schedules.get_input_data(id = 123)
+                >>> c.functions.schedules.get_input_data(id=123)
         """
-        url = f"/functions/schedules/{id}/input_data"
+        url = f"{self._RESOURCE_PATH}/{id}/input_data"
         res = self._get(url)
 
-        return res.json()["data"]
+        return res.json().get("data")
