@@ -406,7 +406,8 @@ class SessionsAPI(APIClient):
             client_credentials (Optional[ClientCredentials]): The client credentials to create the session. If set to None,
                 a session will be created using the credentials used to instantiate -this- CogniteClient object. If that
                 was done using a token, a session will be created using token exchange. Similarly, if the credentials were
-                client credentials, a session will be created using client credentials.
+                client credentials, a session will be created using client credentials. This method does not work when
+                using client certificates (not supported server-side).
 
         Returns:
             CreatedSession: The object with token inspection details.
@@ -424,7 +425,7 @@ class SessionsAPI(APIClient):
             id (Union[int, Sequence[int]): Id or list of session ids
 
         Returns:
-            List of revoked sessions. If the user does not have the sessionsAcl:LIST capability, then only the session IDs will be present in the response.
+            SessionList: List of revoked sessions. If the user does not have the sessionsAcl:LIST capability, then only the session IDs will be present in the response.
         """
         identifiers = IdentifierSequence.load(ids=id, external_ids=None)
         items = {"items": identifiers.as_dicts()}
