@@ -6,9 +6,9 @@ from typing import TYPE_CHECKING, Any, Collection, Dict, Generic, List, Optional
 
 from cognite.client import utils
 from cognite.client.exceptions import CogniteMissingClientError
-from cognite.client.utils._auxiliary import convert_all_keys_to_camel_case, to_camel_case, to_snake_case
 from cognite.client.utils._identifier import IdentifierSequence
 from cognite.client.utils._pandas_helpers import notebook_display_with_fallback
+from cognite.client.utils._text import convert_all_keys_to_camel_case, to_camel_case, to_snake_case
 from cognite.client.utils._time import convert_time_attributes_to_datetime
 
 if TYPE_CHECKING:
@@ -213,6 +213,16 @@ class CogniteResourceList(UserList):
         item = convert_time_attributes_to_datetime(self.dump())
         return json.dumps(item, default=utils._auxiliary.json_dump_default, indent=4)
 
+    # TODO: We inherit a lot from UserList that we don't actually support...
+    def extend(self, other: Collection[Any]) -> None:  # type: ignore [override]
+        other_res_list = type(self)(other)  # See if we can accept the types
+        if set(self._id_to_item).isdisjoint(other_res_list._id_to_item):
+            super().extend(other)
+            self._external_id_to_item.update(other_res_list._external_id_to_item)
+            self._id_to_item.update(other_res_list._id_to_item)
+        else:
+            raise ValueError("Unable to extend as this would introduce duplicates")
+
     def dump(self, camel_case: bool = False) -> List[Dict[str, Any]]:
         """Dump the instance into a json serializable Python data type.
 
@@ -324,12 +334,13 @@ class CogniteUpdate:
         update_obj["modify"] = value
         self._update_object[name] = update_obj
 
-    def dump(self) -> Dict[str, Any]:
+    def dump(self, camel_case: bool = True) -> Dict[str, Any]:
         """Dump the instance into a json serializable Python data type.
 
         Returns:
             Dict[str, Any]: A dictionary representation of the instance.
         """
+        assert camel_case is True, "snake_case is currently unsupported"  # TODO maybe (when unifying classes)
         dumped: Dict[str, Any] = {"update": self._update_object}
         if self._id is not None:
             dumped["id"] = self._id
