@@ -8,7 +8,7 @@ from cognite.client.data_classes._base import (
     CogniteResource,
     CogniteResourceList,
 )
-from cognite.client.utils._auxiliary import convert_all_keys_to_camel_case, to_camel_case
+from cognite.client.utils._text import convert_all_keys_to_camel_case, to_camel_case
 
 if TYPE_CHECKING:
     from cognite.client import CogniteClient
