@@ -3,7 +3,8 @@ from __future__ import annotations
 from typing import Any, Dict, List, Literal, Union, cast
 
 from cognite.client.data_classes._base import CognitePropertyClassUtil
-from cognite.client.utils._auxiliary import convert_all_keys_to_camel_case, handle_deprecated_camel_case_argument
+from cognite.client.utils._auxiliary import handle_deprecated_camel_case_argument
+from cognite.client.utils._text import convert_all_keys_to_camel_case
 
 
 class TimestampRange(dict):
